package wash.control;

import actor.ActorThread;
import wash.io.WashingIO;
import wash.control.WashingMessage.Order;

public final class ControllerTemp extends ActorThread<WashingMessage> {
    private WashingIO io;
    private Heater heater;
    private WashingMessage m;

    public ControllerTemp(WashingIO io) {
        this.io = io;
        heater = new Heater(0);
    }

    protected void sendAck() {
        m.sendAck(this);
    }

    @Override
    public void run() {
        heater.start();

        while (true) {
            try {
                m = take();
            } catch (Exception e) {
                System.exit(1);
            }

            switch (m.order()) {
                case Order.TEMP_SET_40 -> heater.setTarget(40);
                case Order.TEMP_SET_60 -> heater.setTarget(60);
                case Order.TEMP_IDLE -> heater.setTarget(0); // TODO: Error
                default -> {
                    continue;
                }
            }
        }
    }

    /**
     * Heater class that extends Thread and controls the temperature
     * of the washing machine.
     *
     * Note that the heater has access to local variables in the
     * TemperatureController class.
     */
    class Heater extends Thread {
        double target;
        boolean hovering;

        Heater(double target) {
            this.target = target;
            this.hovering = true; // No acks before actual setpoint is acquired
        }

        synchronized public void setTarget(double target) {
            this.target = target;
            this.hovering = false;
        }

        @Override
        public void run() {
            double current;
            try {
                while (!isInterrupted()) {
                    current = io.getTemperature();

                    if (io.getWaterLevel() == 0) {
                        io.heat(false);
                        sleep(60000 / Settings.SPEEDUP);
                        continue;
                    }

                    if (current < target) {
                        io.heat(true);
                    }

                    else if (current >= target) {
                        io.heat(false);

                        if (!hovering) {
                            sendAck();
                            hovering = true;
                            sleep(60000 / Settings.SPEEDUP);
                            continue;
                        }
                    }

                    sleep(60000 / Settings.SPEEDUP);
                }
            } catch (InterruptedException e) {
                io.heat(false);
                Thread.currentThread().interrupt();
            }
        }
    }
}
